package vsp.uloha437;

import cz.zcu.fav.kiv.jsim.JSimInvalidParametersException;
import cz.zcu.fav.kiv.jsim.JSimMethodNotSupportedException;
import cz.zcu.fav.kiv.jsim.JSimSecurityException;
import cz.zcu.fav.kiv.jsim.JSimSimulation;
import cz.zcu.fav.kiv.jsim.JSimSimulationAlreadyTerminatedException;
import cz.zcu.fav.kiv.jsim.JSimTooManyHeadsException;
import cz.zcu.fav.kiv.jsim.JSimTooManyProcessesException;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * 1. uloha VSP
 * 4. priklad: Generovani nahodnych cisel - exponencialni rozdeleni
 *
 * Hlavni spousteci trida.
 *
 * @author Jiri Kucera (kalwi@students.zcu.cz)
 */
public class Main {

    /**
     * Vychozi hodnota parametru exponencialniho rozdeleni doby mezi vstupnimi pozadavky.
     */
    public static final double DEFAULT_LAMBDA = 4;

    /**
     * Vychozi pocet vygenerovanych pozadavku.
     */
    public static final int DEFAULT_NO_TRANSACTIONS = 10000;

    /**
     * Pravdepodobnost, s jakou budou pozadavky posilany do fronty prvniho serveru.
     */
    public static final double P1 = 0.8;

    /**
     * Parametr exponencialniho rozdeleni doby zpracovani pozadavku v prvnim serveru.
     */
    public static final double MI1 = 4;

    /**
     * Parametr exponencialniho rozdeleni doby zpracovani pozadavku v druhem serveru.
     */
    public static final double MI2 = 80;

    /**
     * Kontruktor tridy, obsahuje definici struktury site a spousti simulaci.
     * 
     * @param count Pocet vstupnich pozadavku.
     * @param lambda Parametr exponencialniho rozdeleni doby mezi vstupnimi pozadavky.
     */
    public Main(int count, double lambda) {
	JSimSimulation simulation = null;

	try {
	    System.out.println("pocet vstupnich pozadavku: " + count);
	    System.out.println("parametr exp. rozdeleni vstupnich pozadavaku: " + lambda);

	    System.out.println("\nsimulace spustena...\n");

	    simulation = new JSimSimulation("kalwi");

	    Terminator t = new Terminator();

	    Server server1 = new Server("server1", simulation, MI1, t);
	    Server server2 = new Server("server2", simulation, MI2, t);

	    Junction queue1 = server1.getQueue();
	    Junction queue2 = server2.getQueue();

	    Fork crossing = new Fork(queue1, queue2, P1);

	    Generator generator = new Generator("generator", simulation, lambda, count, crossing);

	    generator.activateNow();

	    while(simulation.step()) {
	    }

	    System.out.println("hotovo...\n");

	    double lq = server1.getLq() + server2.getLq();

	    System.out.println("Lq: " + lq);
	    System.out.println("Tq: " + lq / lambda);
	    
	    System.out.println("druha fronta prazdna (%): " + (server2.getQueue().getEmptyTime() * 100 / simulation.getCurrentTime()));

	} catch (JSimSecurityException ex) {
	    Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
	} catch (JSimTooManyHeadsException ex) {
	    Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
	} catch (JSimMethodNotSupportedException ex) {
	    Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
	} catch (JSimSimulationAlreadyTerminatedException ex) {
	    Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
	} catch (JSimTooManyProcessesException ex) {
	    Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
	} catch (JSimInvalidParametersException ex) {
	    Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
	} catch (Exception ex) {
	    Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
	} finally {
	    simulation.shutdown();
	}
    }

    /**
     * Spousteci metoda.
     * 
     * @param args Prvni parametr je pocet pozadavku.<br/>
     * Druhy parametr je parametr lambda exponencialniho rozdeleni doby mezi
     * vstupnimi pozadavky.
     */
    public static void main(String[] args) {
	try {
	    if (args.length == 0) {
		printHelp();
		new Main(DEFAULT_NO_TRANSACTIONS, DEFAULT_LAMBDA);
	    } else if (args.length == 1) {
		printHelp();
		new Main(Integer.parseInt(args[0]), DEFAULT_LAMBDA);
	    } else if (args.length == 2) {
		new Main(Integer.parseInt(args[0]), Double.parseDouble(args[1]));
	    } else {
		printHelp();
		new Main(Integer.parseInt(args[0]), Double.parseDouble(args[1]));
	    }
	} catch (NumberFormatException e) {
	    printHelp();
	}
    }

    /**
     * Vypise napovedu.
     */
    private static void printHelp() {
	System.out.println("pouziti: java -jar vsp.jar <pocet> <lambda>\n");
    }
}
