package vsp.uloha437;

import cz.zcu.fav.kiv.jsim.JSimHead;
import cz.zcu.fav.kiv.jsim.JSimInvalidParametersException;
import cz.zcu.fav.kiv.jsim.JSimLink;
import cz.zcu.fav.kiv.jsim.JSimSecurityException;
import cz.zcu.fav.kiv.jsim.JSimSimulation;
import cz.zcu.fav.kiv.jsim.JSimTooManyHeadsException;

/**
 * Fronta serveru. Tato trida nemuze byt instanciovana samostatne,
 * kazdy server si ji vytvori sam.
 *
 * @author Jiri Kucera (kalwi@students.zcu.cz)
 */
public class Queue implements Junction {

    private JSimHead queue;
    private Server server;

    private double emptyTime = 0;
    private double emptyStart = 0;

    /**
     * Konstruktor tridy.
     * @param name Nazev objektu.
     * @param simulation Reference na objekt simulacniho modelu.
     * @param server Server, ktery z fronty bude odebirat pozadavky.
     * @throws JSimInvalidParametersException
     * @throws JSimTooManyHeadsException
     * @throws Exception
     */
    protected Queue(String name, JSimSimulation simulation, Server server) throws JSimInvalidParametersException, JSimTooManyHeadsException, Exception {
	if (server.getQueue() != null) {
	    throw new Exception("Class Queue can not be instantiated separately");
	}
	
	this.server = server;
	this.queue = new JSimHead(name, simulation);
    }

    /**
     * Vraci, zda-li je fronta prazdna.
     * @return true, pokud je fronta prazdna, false, pokud neni prazdna.
     */
    public boolean isEmpty() {
	return queue.empty();
    }

    /**
     * Odebere z fronty pozadavek a vrati jej.
     * @return Pozadavek odebrany z fronty.
     * @throws JSimSecurityException
     */
    public Transaction first() throws JSimSecurityException {
	if (isEmpty()) {
	    return null;
	}
	
	JSimLink link = queue.first();
	link.out();

	if (isEmpty()) {
	    emptyStart = queue.getCurrentTime();
	}

	return (Transaction) link;
    }

    /**
     * Vraci dobu, po kterou byla fronta prazdna.
     * @return Doba, po kterou byla fronta prazdna.
     */
    public double getEmptyTime() {
	return emptyTime;
    }
    
    @Override
    public void put(Transaction tr) throws JSimSecurityException {
	if (isEmpty()) {
	    emptyTime += (queue.getCurrentTime() - emptyStart);
	}
	
	tr.into(queue);

	if (server.isIdle()) {
	    server.activateNow();
	}

    }
}
